#include <UrsEsp.h>
#include <esp_heap_caps.h>
#include <esp32-hal-psram.h>

extern "C" int rtc_get_reset_reason(int cpu_no); // Deklariert in <cpu>/rom/rtc.
                                                 // Die Anzahl der Grnde ist abhnigig vom CPU-Typ.
                                                 // Gleiche Grnde besitzen fr alle CPUs die geleiche Nummer.

namespace UrsCPU {
   int UrsEspClass::getResetReason(int cpuNumber)
 {
      return rtc_get_reset_reason(cpuNumber);
   }

   // Konstante Strings werden beim ESP32 im Flash gespeichert. PROGMEM etc. sind leere Makros.

   String UrsEspClass::getResetReasonName(int reason) {
      switch (reason) {
      case 1: return "POWERON_RESET"; break;          // 1,  Vbat power on reset
      case 3: return "SW_RESET"; break;               // 3,  Software reset digital core
      case 4: return "OWDT_RESET"; break;             // 4,  Legacy watch dog reset digital core
      case 5: return "DEEPSLEEP_RESET"; break;        // 5,  Deep Sleep reset digital core
      case 6: return "SDIO_RESET"; break;             // 6,  Reset by SLC module, reset digital core
      case 7: return "TG0WDT_SYS_RESET"; break;       // 7,  Timer Group0 Watch dog reset digital core
      case 8: return "TG1WDT_SYS_RESET"; break;       // 8,  Timer Group1 Watch dog reset digital core
      case 9: return "RTCWDT_SYS_RESET"; break;       // 9,  RTC Watch dog Reset digital core
      case 10: return "INTRUSION_RESET"; break;       // 10, Instrusion tested to reset CPU
      case 11: return "TGWDT_CPU_RESET"; break;       // 11, Time Group reset CPU
      case 12: return "SW_CPU_RESET"; break;          // 12, Software reset CPU
      case 13: return "RTCWDT_CPU_RESET"; break;      // 13, RTC Watch dog Reset CPU
      case 14: return "EXT_CPU_RESET"; break;         // 14, for APP CPU, reseted by PRO CPU
      case 15: return "RTCWDT_BROWN_OUT_RESET"; break;// 15, Reset when the vdd voltage is not stable
      case 16: return "RTCWDT_RTC_RESET"; break;      // 16, RTC Watch dog reset digital core and rtc module
      case 17: return "TG1WDT_CPU_RESET"; break;      // 17, Time Group1 reset CPU
      case 18: return "SUPER_WDT_RESET"; break;      // 18, super watchdog reset digital core and rtc module
      case 19: return "GLITCH_RTC_RESET"; break;      // 19, glitch reset digital core and rtc module
      case 20: return "EFUSE_RESET"; break;           // 20, efuse reset digital core
      case 21: return "USB_UART_CHIP_RESET"; break;   // 21, usb uart reset digital core
      case 22: return "USB_JTAG_CHIP_RESET"; break;   // 22, usb jtag reset digital core
      case 23: return "POWER_GLITCH_RESET"; break;    // 23, power glitch reset digital core and rtc module
      case 24: return "JTAG_RESET"; break;            // 24, jtag reset CPU



      default: return "NO_MEAN";
      }
   }
   String UrsEspClass::getResetReasonDescription(int reason) {
      switch (reason) {
      case 1: return "Vbat power on reset"; break;
      case 3: return "Software reset digital core"; break;
      case 4: return "Legacy watch dog reset digital core"; break;
      case 5: return "Deep Sleep reset digital core"; break;
      case 6: return "Reset by SLC module, reset digital core"; break;
      case 7: return "Timer Group0 Watch dog reset digital core"; break;
      case 8: return "Timer Group1 Watch dog reset digital core"; break;
      case 9: return "RTC Watch dog Reset digital core"; break;
      case 10: return "Instrusion tested to reset CPU"; break;
      case 11: return "Time Group reset CPU"; break;
      case 12: return "Software reset CPU"; break;
      case 13: return "RTC Watch dog Reset CPU"; break;
      case 14: return "For APP CPU, reseted by PRO CPU"; break;
      case 15: return "Reset when the vdd voltage is not stable"; break;
      case 16: return "RTC Watch dog reset digital core and rtc module"; break;
      case 17: return "Time Group1 reset CPU"; break;
      case 18: return "Super watchdog reset digital core and rtc module"; break;
      case 19: return "Glitch reset digital core and rtc module"; break;
      case 20: return "EFUSE reset digital core"; break;
      case 21: return "USB UART reset digital core"; break;
      case 22: return "USB JTAG reset digital core"; break;
      case 23: return "Power glitch reset digital core and rtc module"; break;
      case 24: return "JTAG reset CPU"; break;
      default: return "NO_MEAN";
      }
   }

   int UrsEspClass::registerShutdownHandler(void(*shutdownHandler)(void)) {
      return esp_register_shutdown_handler(shutdownHandler);
   }

   int UrsEspClass::unregisterShutdownHandler(void(*shutdownHandler)(void)) {
      return esp_unregister_shutdown_handler(shutdownHandler);
   }

   uint32_t UrsEspClass::getLargestFreeDmaHeapBlock() {
      return heap_caps_get_largest_free_block(MALLOC_CAP_DMA);
   }

   void* UrsEspClass::allocDmaHeapBlock(size_t size) {
      return heap_caps_malloc(size, MALLOC_CAP_DMA);
   }

   void* UrsEspClass::reallocDmaHeapBlock(void* ptr, size_t newSize) {
      return heap_caps_realloc(ptr, newSize, MALLOC_CAP_DMA);
   }

   bool UrsEspClass::psramFound() {
       ::psramFound();
   }

   void* UrsEspClass::psramMalloc(size_t size) {
      return  ps_malloc( size);
   }

   void* UrsEspClass::psramRealloc(void* ptr, size_t newSize) {
      return ps_realloc(ptr, newSize);
   }
   
   UrsEspClass UrsESP;
}